from django.http import HttpResponseRedirect
from django.views.decorators.csrf import csrf_exempt
from django.core.exceptions import PermissionDenied

from georemindme.decorators import login_required

from models import AccessToken
from OAuthConsumerClientFactory import OAuthConsumerClientFactory
import OAuthRequest
from appengine_utilities.sessions import Session

@login_required
def request_auth(request, provider):
    #load consumer
    try:
        consumer, client = OAuthConsumerClientFactory.getInstances(provider=provider)
    except:
        raise PermissionDenied()
    try:
        oauth_user = AccessToken.objects.get_user_token(user=request.session['user'], provider=provider)
        oauth_user.delete()
    except:
        pass
    
    oauth_user = AccessToken(user=request.session['user'].key(), provider=provider)
  
    orequest = OAuthRequest.OAuthRequest.get_request_token(oauth_consumer=consumer, callback=client.callback_url, http_url=client.request_token_url)
    token = client.fetch_request_token(orequest)
    if not token:
        raise PermissionDenied()
    
    oauth_user.token_key = token.key
    oauth_user.token_secret = token.secret
    oauth_user.put()
        
    orequest = OAuthRequest.OAuthRequest.authorize_request_token(token=token, callback=client.callback_url, http_url=client.authorization_url)
    return HttpResponseRedirect(client.get_token_authorize_url(orequest))

@csrf_exempt
def authorized_auth(request, provider):
    try:
        consumer, client = OAuthConsumerClientFactory.getInstances(provider=provider)
    except:
        raise PermissionDenied() 
    #provider sends the verifier for a request token
    try:
        oauth_token = request.GET.get('oauth_token')
        oauth_verifier = request.GET.get('oauth_verifier')
        
        oauth_user = AccessToken.objects.get_token_key(str(oauth_token), provider=provider)
        oauth_user.verifier = str(oauth_verifier)
        
        #send the request token to get a new authorized access token
        token = OAuthRequest.OAuthToken(oauth_user.token_key, oauth_user.token_secret, oauth_user.verifier)
        request = OAuthRequest.OAuthRequest.get_from_provider(oauth_consumer=consumer, token=token,
                                                              http_url=client.authorization_url,
                                                              )
        #save the authorized token
        token = client.fetch_access_token(request)
    
        oauth_user.token_key = token.key
        oauth_user.token_secret = token.secret
        oauth_user.authorized_access = True
        oauth_user.put()
    
        return HttpResponseRedirect('/')
    except:#no authorization
        raise PermissionDenied()